import java.util.*;
import java.io.*;

public class Marker {

    private ArrayList<Person> members;

    public Marker(){
	members = new ArrayList<Person>();
    }

    public void addMembers(String namesFile) throws IOException {
	Scanner sc = new Scanner(new File(namesFile));
	while (sc.hasNext()) add(sc.next());
	sc.close();
    }
    //
    // namesFile is name of a file of names of people
    //

    private void add(String name){members.add(new Person(name));}
    //
    // add a person to Marker
    //

    public void addFriends(String friendsFile) throws IOException {
	Scanner sc = new Scanner(new File(friendsFile));
	while (sc.hasNext()) makeFriends(find(sc.next()),find(sc.next()));
	sc.close();
    }

    private static int binSearch(String name,ArrayList<Person> data,int low,int high){
	if (low > high) return -1;
	int mid = (low + high) / 2;
	int c   = name.compareTo(data.get(mid).getName());
	if (c < 0) return binSearch(name,data,low,mid-1);
	else if (c == 0) return mid;
	else return binSearch(name,data,mid+1,high);
    }
    //
    // returns the location of the person with given name
    //

    public Person find(String name){
	int i = binSearch(name,members,0,members.size()-1);
	if (i == -1) return null;
	return members.get(i);
    }
    //
    // IMPLEMENT THIS
    // find, and return, the Person with this
    // name or null if not present
    //

     private void makeFriends(Person p1,Person p2){
	 p1.addFriend(p2);
	 p2.addFriend(p1);
     }
    //
    // IMPLEMENT THIS
    // make person p1 and person p2 friends
    //

    public boolean areFriends(Person p1,Person p2){
	if (p1 == null || p2 == null) return false;
	for (Person p : p1.getFriends()) if (p == p2) return true;
	return false;
    }
    //
    // IMPLEMENT THIS
    // given two people, are they friends?
    //

    public ArrayList<Person> commonFriends(Person p1,Person p2){
	if (p1 == null || p2 == null) return null;
	ArrayList<Person> cf = new ArrayList<Person>();
	for (Person p : p1.getFriends()) if (areFriends(p,p2)) cf.add(p);
	return cf;
    }
    //
    // IMPLEMENT THIS
    // return the friends shared by two people
    //

    public Person leastPopular(){
	int popularity = members.size()+1; 
	Person leastPopular = null;
	int size;
	for (Person p : members){
	    size = p.getFriends().size();
	    if (size < popularity){popularity = size; leastPopular = p;}
	}
	return leastPopular;
    }
    //
    // IMPLEMENT THIS
    // find the least popular repson
    //

    public Person mostPopular(){
	int popularity = -1;
	Person mostPopular = null;
	int size;
	for (Person p : members){
	    size = p.getFriends().size();
	    if (size > popularity){popularity = size; mostPopular = p;}
	}
	return mostPopular;
    }
    //
    // find the most popular person
    //

    public int avgPopularity(){
	int sum = 0;
	for (Person p : members) sum = sum + p.getFriends().size();
	return sum / members.size();
    }
    //
    // IMPLEMENT THIS
    // return the average popularity
    //

    public int size(){
	return members.size();
    }
    //
    // IMPLEMENT THIS
    // return the number of people in Marker
    //

    private static boolean equalPeople(ArrayList<Person> p1,ArrayList<Person> p2){
	for (Person x : p1){
	    boolean found = false;
	    for (Person y : p2) if (x.getName().equals(y.getName())) found = true;
	    if (!found) return false;
	}
	return p1.size() == p2.size();
    }

    public static boolean equal(ArrayList<Person> p1,ArrayList<Person> p2){
	return equalPeople(p1,p2) && equalPeople(p2,p1);
    }

    public static void main(String[] args) throws Exception {
	System.out.println("ex00 marker (version 6.0021415 x 10^23)");
	String members = "members.txt";
	String friends = "friends05.txt";
	int mark = 0;
	int totalMarks = 0;

	ArrayList<String> names = new ArrayList<String>();
	Scanner sc = new Scanner(new File(members));
	while (sc.hasNext()) names.add(sc.next());
	sc.close();

	System.out.println("test1: create your FBook");
	Marker fb0 = new Marker();
	FBook  fb1 = new FBook();
	fb0.addMembers(members);
	fb1.addMembers(members);
	fb0.addFriends(friends);
	fb1.addFriends(friends);

	System.out.println("test2: checking members");
	boolean memberTest = true;
	for (String name : names){
	    Person p1 = fb0.find(name);
	    Person p2 = fb1.find(name);
	    if (p1 == null || p2 == null || !p1.getName().equals(p2.getName()))
		memberTest = false;
	}
	System.out.println("  test2: member test passed?: "+ memberTest);

	System.out.println("test3: checking friends");
	boolean friendTest = true;
	for (String name : names){
	    Person p1 = fb0.find(name);
	    Person p2 = fb1.find(name);
	    if (p1 == null || p2 == null || !equal(p1.getFriends(),p2.getFriends()))
		friendTest = false;
	}
	System.out.println("  test3: friends test passed?: "+ friendTest);

	System.out.println("test4: checking find");
	boolean findTest = true;
	for (String name : names){
	    Person p1 = fb0.find(name);
	    Person p2 = fb1.find(name);
	    Person p11 = fb0.find(name + "xxx");
	    Person p22 = fb1.find(name + "xxx");
	    if (p2 == null || p22 != null || !p1.getName().equals(p2.getName()))
		findTest = false;
	}
	System.out.println("  test5: find test passed?: "+ findTest);

	System.out.println("test5: checking popularity");
	boolean popularityTest = true;
	Person mostP1 = fb0.mostPopular();
	Person mostP2 = fb1.mostPopular();
	Person leastP1 = fb0.leastPopular();
	Person leastP2 = fb1.leastPopular();
	popularityTest = mostP2 != null && leastP2 != null &&
	                 mostP1.getFriends().size() == mostP2.getFriends().size() &&
	                 leastP1.getFriends().size() == leastP2.getFriends().size() &&
	                 fb0.avgPopularity() == fb1.avgPopularity();	                
	System.out.println("  test5: popularity test passed?: "+ popularityTest);

	System.out.println("test6: checking size");
	boolean sizeTest = fb0.size() == fb1.size();
	System.out.println("  test6: size test passed?: "+ popularityTest);
    }
}
